# This file is part of the graph-includes package
#
# (c) 2005 Yann Dirson <ydirson@altern.org>
# Distributed under version 2 of the GNU GPL.

package graphincludes::project::default;
use strict;
use warnings;

use base qw(graphincludes::project);

use File::Basename;
use graphincludes::params;

sub new {
  my $class = shift;
  my $self;

  $self  = $class->SUPER::new(@_);

  bless ($self, $class);

  $self->{IGNOREDDEPS} = $self->ignored_deps();
  $self->{IGNOREDEDGES} = {};	# to be computed in getdeps

  return $self;
}

sub filelabel {
  my $self = shift;
  my ($file,$level) = @_;
  $level = $graphincludes::params::minshow unless defined $level;

  $file =~ s/^$self->{PFXSTRIP}// if defined $self->{PFXSTRIP};
  if ($level == 0) {
    return $file;
  } elsif ($level == 1) {
    $file =~ s/\.[^.]*$//;
    return $file;
  } elsif ($level == 2) {
    if ($file =~ m!(.*)/.*!) {
      return $1;
    } else {
      return '<' . $self->filelabel($file, $level - 1) . '>';
    }
  }
  return undef;
}

sub defaultcolors {
  return ();
}

sub ignored_deps {
  return {};
}

sub special_edge {
  my $self = shift;
  my ($src, $dst) = @_;

  my $lbl = $self->{IGNOREDEDGES}->{$src}->{$dst};

  if (defined $lbl) {
    return "color=gray,constraint=false,label=\"$lbl\"";
  } else {
    return $self->SUPER::special_edge($src,$dst);
  }
}

sub _fileexists {
  my ($file, @path) = @_;
  foreach my $dir (@path) {
    my $f = "$dir/$file";
    return $f if -r $f;
  }
  return undef;
}

sub getdeps {
  my $self = shift;
  my %deps;

  while (<>) {
    my $dstfile;
    if (m/^\s*#\s*include\s*"(.*)"/) {
      $dstfile = $self->locatefile ($1, dirname($ARGV), @graphincludes::params::inclpath);
    } elsif (m/^\s*#\s*include\s*<(.*)>/) {
      $dstfile = $self->locatefile ($1, @graphincludes::params::inclpath);
    } else {
      next;
    }

    if (defined $dstfile) {
      my $orignode = $self->{NODEIDS}->{$self->filelabel($ARGV)};
      my $destnode = $self->{NODEIDS}->{$self->filelabel($dstfile)};
      if (defined $self->{IGNOREDDEPS}->{$ARGV}->{$dstfile}) {
	print STDERR "ignoring $ARGV -> $dstfile\n" if $graphincludes::params::debug;
	$self->{IGNOREDEDGES}->{$orignode}->{$destnode} =
	  $self->{IGNOREDDEPS}->{$ARGV}->{$dstfile};
      }
      push (@{$deps{$orignode}}, $destnode);
    } elsif (defined _fileexists ($1, @graphincludes::params::sysinclpath)) {
      $self->{REPORT}->{SYS}->{$1} = 1;
    } else {
      push @{$self->{REPORT}->{HDR}->{$1}}, $ARGV;
    }
  }
  return \%deps;
}

1;
